#include <iostream>
#include <fstream>
#include <cstdlib>
#include <algorithm>

using namespace std;

class Treap {
  public:
    Treap() {
        nil_ = new Node(0, -1);
        nil_->left = nil_->right = nil_;
        nil_->count = nil_->sum = nil_->lowest_sum = 0;
        root_ = nil_;
    }

    long long sum() const {
        return root_->sum;
    }

    int size() const {
        return root_->count;
    }

    void insert(int where, int value) {
        root_ = insert(root_, where, value, rand());
    }

    void erase(int where) {
        root_ = erase(root_, where);
    }

    int whereMin() {
        return whereMin(root_);
    }

  private:
    class Node {
      public:
        Node(int _value, int _priority):
            value(_value),
            priority(_priority) {
        }

        Node(int _value, int _priority, Node* _left, Node* _right):
                value(_value),
                priority(_priority),
                left(_left),
                right(_right) {
            update();
        }

        void update() {
            count = left->count + right->count + 1;
            sum = left->sum + right->sum + value;

            lowest_sum = left->lowest_sum;
            if (left->sum + value < lowest_sum)
                lowest_sum = left->sum + value;
            if (left->sum + value + right->lowest_sum < lowest_sum)
                lowest_sum = left->sum + value + right->lowest_sum;
        }

        int value, priority;

        Node *left, *right;

        int count;
        long long sum, lowest_sum;
    };

    class PairNode {
      public:
        PairNode(Node* _left, Node* _right):
            left(_left),
            right(_right) {
        }

        Node *left, *right;
    };

    Node* insert(Node* root, int where, int value, int priority) {
        if (root->priority < priority) {
            PairNode current = split(root, where);
            return new Node(value, priority, current.left, current.right);
        }

        if (where <= root->left->count)
            root->left = insert(root->left, where, value, priority);
        else
            root->right = insert(root->right, where - root->left->count - 1, value, priority);

        root->update();
        return root;
    }

    PairNode split(Node* root, int where) {
        if (root == nil_)
            return PairNode(nil_, nil_);
        if (where <= root->left->count) {
            PairNode partial_split = split(root->left, where);
            root->left = partial_split.right;
            root->update();
            return PairNode(partial_split.left, root);
        }

        PairNode partial_split = split(root->right, where - root->left->count - 1);
        root->right = partial_split.left;
        root->update();
        return PairNode(root, partial_split.right);
    }

    Node* erase(Node* root, int where) {
        if (where == root->left->count + 1) {
            Node *answer = merge(root->left, root->right);
            delete root;
            return answer;
        }

        if (where <= root->left->count) {
            root->left = erase(root->left, where);
            root->update();
            return root;
        }

        root->right = erase(root->right, where - root->left->count - 1);
        root->update();
        return root;
    }

    Node* merge(Node* left, Node* right) {
        if (left == nil_ && right == nil_)
            return nil_;

        if (left->priority < right->priority) {  // right is root
            right->left = merge(left, right->left);
            right->update();
            return right;
        }

        left->right = merge(left->right, right);
        left->update();
        return left;
    }

    int whereMin(Node* root) {
        if (root == nil_)
            return 0;

        if (root->left->sum + root->value == root ->lowest_sum)
            return root->left->count + 1;

        if (root->left->lowest_sum == root->lowest_sum)
            return whereMin(root->left);

        return root->left->count + 1 + whereMin(root->right);
    }

    Node *root_, *nil_;
};

int main() {
    ifstream cin("rotatii.in");
    ofstream cout("rotatii.out");

    srand(time(NULL));
    int N, M; cin >> N >> M;
    Treap T;
    for (int i = 0; i < N; ++i) {
        int value; cin >> value;
        T.insert(i, value);
    }

    for (int i = 0; i < M; ++i) {
        int type; cin >> type;
        if (type == 0) {
            int where, value;
            cin >> where >> value;
            T.insert(where, value);
        } else if (type == 1) {
            int where;
            cin >> where;
            T.erase(where);
        } else {
            if (T.sum() < 0)
                cout << "-1\n";
            else
                cout << T.whereMin() % T.size() << "\n";
        }
    }
}
