#include <iostream>
#include <fstream>
#include <iostream>
#include <vector>

using namespace std;

class FenwickTree {
  public:
    explicit FenwickTree(const size_t &size, const int &value):
            data_(size + 1, 0) {
        for (size_t i = 1; i <= size; ++i)
            data_[i] = (i & -i) * value;
    }

    void add(size_t where, size_t value) {
        for (;where < data_.size(); where += (where & -where))
            data_[where] += value;
    }

    int find(int value) {
        size_t where = 0, step;
        for (step = 1; step < data_.size(); step *= 2);
        for (; step; step /= 2)
            if (where + step < data_.size() && data_[where + step] < value) {
                where += step;
                value -= data_[where];
            }
        return where + 1;
    }

    int query(size_t where) {
        int answer = 0;
        for (; where; where -= (where & -where))
            answer += data_[where];
        return answer;
    }

  private:
      vector<int> data_;
};

typedef long long int64;

class SegmentTree {
  public:
    explicit SegmentTree(size_t size) {
        while (size & (size - 1))
            size += (size & -size);

        size_ = size;

        sum_ = new int64[2 * size];
        best_ = new int64[2 * size];
        for (size_t i = 0; i < size; ++i)
            sum_[i] = best_[i] = 0;
    }

    ~SegmentTree() {
        delete[] sum_;
        delete[] best_;
    }

    int64 sum() {
        return sum_[1];
    }

    void add(int where, int value) {
        where += size_;
        sum_[where] = best_[where] = value;
        
        for (where /= 2; where; where /= 2) {
            sum_[where] = sum_[where * 2] + sum_[where * 2 + 1];
            best_[where] = min(best_[where * 2], sum_[where * 2] + best_[where * 2 + 1]);

        }
    }

    int goodRotation() {
        size_t where = 1;
        while (where < size_) {
            if (best_[where * 2] <= sum_[where * 2] + best_[where * 2 + 1])
                where = where * 2;
            else
                where = where * 2 + 1;
        }

        return where - size_;
    }

  private:
    size_t size_;
    int64 *sum_;
    int64 *best_;
};


int main() {
    ifstream cin("rotatii.in");
    ofstream cout("rotatii.out");

    int N, M; cin >> N >> M;

    vector< pair<int, int> > Q(N + M);
    for (int i = 0; i < N + M; ++i) {
        if (i < N) {
            Q[i].first = i + 1;
            cin >> Q[i].second;
            continue;
        }

        int type; cin >> type;
        if (type == 0) {
            cin >> Q[i].first >> Q[i].second;
            ++Q[i].first;
        } else
            Q[i] = make_pair(-1, -1);
    }

    N += M;
    FenwickTree T(N + 1, 1);
    for (int i = N - 1; i >= 0; --i)
        if (Q[i].first != -1) {
            Q[i].first = T.find(Q[i].first);
            T.add(Q[i].first, -1);
        }

    SegmentTree S(N + 1);
    for (int i = 0; i < N; ++i) {
        if (Q[i].first == -1) {
            if (S.sum() < 0)
                cout << "-1\n";
            else {
                int answer = S.goodRotation();
                cout << T.query(answer) << "\n";
            }
        } else {
            S.add(Q[i].first, Q[i].second);
            T.add(Q[i].first, 1);
        }
    }
}
