#include <iostream>
#include <fstream>
#include <cstdio>
#include <vector>

using namespace std;

class Reader {
  public:
    Reader(const char * const file):
            stream_(fopen(file, "r")) {
        buffer_ = new char[kBufferSize];
        reset();
    }

    Reader& operator>>(int &value) {
        value = 0;
        while ((current() < '0' || current() > '9') && current() != '-')
            next();

        bool positive = true;
        if (current() == '-') {
            positive = false;
            next();
        }

        while (current() >= '0' && current() <= '9') {
            value = value * 10 + current() - '0';
            next();
        }

        if (not positive)
            value = -value;
        return *this;
    }

    ~Reader() {
        delete buffer_;
        fclose(stream_);
    }

  private:
    static const int kBufferSize = 32768;

    char current() const {
        return buffer_[position_];
    }

    void next() {
        if (++position_ == kBufferSize)
            reset();
    }

    void reset() {
        position_ = 0;
        fread(buffer_, 1, kBufferSize, stream_);
    }

    int position_;
    char *buffer_;
    FILE *stream_;
};

typedef long long int64;

class Array {
  public:
    Array(int groups):
            groups_(groups) {
        values_ = new int*[groups];
        sums_ = new int64*[groups];

        values_[0] = new int[groups * groups * 2];
        sums_[0] = new int64[groups * groups * 2];

        for (int i = 1; i < groups; ++i) {
            values_[i] = values_[i - 1] + groups * 2;
            sums_[i] = sums_[i - 1] + groups * 2;
        }

        for (int i = 0; i < groups * groups * 2; ++i)
            values_[0][i] = 0, sums_[0][i] = 0;

        sizes_ = new int[groups];
        best_sums_ = new int64[groups];
        for (int i = 0; i < groups; ++i)
            sizes_[i] = best_sums_[i] = 0;
    }

    void addElement(int where, int value) {
        int group = 0;
        int origwhere = where;
        while (sizes_[group] < where)
            where -= sizes_[group++];
        if (sizes_[group] == groups_ * 2) {  // it's full, noooooooooooooo
            if (where == groups_ * 2) {
                where = 0;
                while (group < groups_ and sizes_[group] == groups_ * 2)
                    ++group;
            } else
                group = groups_;
            if (group == groups_) {
                reset();
                addElement(origwhere, value);
                return;
            }
        }

        for (int i = sizes_[group]; i > where; --i)
            values_[group][i] = values_[group][i - 1];
        values_[group][where] = value;
        ++sizes_[group];

        updatePartialSums(group);
    }

    void eraseElement(int where) {
        // search the group
        int group = 0;
        while (sizes_[group] < where)
            where -= sizes_[group++];

        --sizes_[group];
        --where;
        for (int i = where; i < sizes_[group]; ++i)
            values_[group][i] = values_[group][i + 1];

        updatePartialSums(group);
    }

    int bestRotation() {
        int64 best = 0, toadd = 0;
        int where = 0;
        // find the group first
        for (int i = 0; i < groups_; ++i) {
            if (toadd + best_sums_[i] < best) {
                best = toadd + best_sums_[i];
                where = i;
            }

            if (sizes_[i] > 0)
                toadd += sums_[i][sizes_[i] - 1];
        }

        if (toadd < 0)
            return -1;
        int group = where;
        best = 0, where = 0;
        for (int i = 0; i < sizes_[group]; ++i)
            if (sums_[group][i] < best) {
                best = sums_[group][i];
                where = i + 1;
            }

        // answer is where + total before it
        for (int i = 0; i < group; ++i)
            where += sizes_[i];
        return where;
    }

  private:
    void reset() {
        int total = 0;
        for (int i = 0; i < groups_; ++i)
            total += sizes_[i];

        int *new_sizes = new int[groups_];
        for (int i = 0; i < groups_; ++i) {
            new_sizes[i] = total / groups_;
            if (total % groups_) {
                ++new_sizes[i];
                --total;
            }
        }

        int **new_values = new int*[groups_];
        new_values[0] = new int[groups_ * groups_ * 2];
        for (int i = 1; i < groups_; ++i)
            new_values[i] = new_values[i - 1] + groups_ * 2;

        for (int group = 0, group_new = 0, i = 0, j = 0; group < groups_;) {
            new_values[group_new][j] = values_[group][i];

            if (++i == sizes_[group]) {
                ++group;
                while (group < groups_ && sizes_[group] == 0)
                    ++group;
                i = 0;
            }

            if (++j == new_sizes[group_new]) {
                ++group_new;
                while (group_new < groups_ && new_sizes[group_new] == 0)
                    ++group_new;
                j = 0;
            }
        }

        swap(sizes_, new_sizes);
        swap(values_, new_values);

        delete[] new_values[0];
        delete[] new_values;

        for (int i = 0; i < groups_; ++i)
            updatePartialSums(i);
    }

    void updatePartialSums(int group) {
        best_sums_[group] = 0;
        for (int i = 0; i < sizes_[group]; ++i) {
            sums_[group][i] = values_[group][i];
            if (i > 0)
                sums_[group][i] += sums_[group][i - 1];

            if (sums_[group][i] < best_sums_[group])
                best_sums_[group] = sums_[group][i];
        }
    }

    int groups_;

    int **values_;
    int64** sums_;

    int *sizes_;
    int64* best_sums_;
};


int main() {
    ofstream cout("rotatii.out");

    Reader cin("rotatii.in");
    int N, M; cin >> N >> M;

    int groups = 0;
    for (; groups * groups < N + M; ++groups);

    Array A(groups);
    for (int i = 0; i < N; ++i) {
        int value; cin >> value;
        A.addElement(i, value);
    }

    for (int i = 0; i < M; ++i) {
        int type; cin >> type;
        if (type == 0) {
            int where, value; cin >> where >> value;
            A.addElement(where, value);
        } else if (type == 1) {
            int where; cin >> where;
            A.eraseElement(where);
        } else {
            cout << A.bestRotation() << "\n";
        }
    }
}
